'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var PassportStrategy = require('passport-strategy');
var jwt = require('jsonwebtoken');

var _require = require('util'),
    promisify = _require.promisify;

var lookup = require('./lookup');

var MagicLinkStrategy = function (_PassportStrategy) {
  _inherits(MagicLinkStrategy, _PassportStrategy);

  /**
   * @param {string} secret - The secret to sign the token
   * @param {Array} userFields - An array of mandatory user fields from the request
   * @param {Array} tokenField - The token field from the request
   * @param {Number} ttl - Time to live
   * @param {Boolean} passReqToCallbacks - Whether the req should be passed do the sendToken callback
   * @param {Boolean} verifyUserAfterToken - Whether the user should be verified after the token verification
   * @param {Function} sendToken - A function to deliver the token
   * @param {Function} verifyUser - A function to verify the user
   */
  function MagicLinkStrategy(_ref, sendToken, verifyUser) {
    var secret = _ref.secret,
        userFields = _ref.userFields,
        tokenField = _ref.tokenField,
        _ref$ttl = _ref.ttl,
        ttl = _ref$ttl === undefined ? 60 * 10 : _ref$ttl,
        _ref$passReqToCallbac = _ref.passReqToCallbacks,
        passReqToCallbacks = _ref$passReqToCallbac === undefined ? false : _ref$passReqToCallbac,
        _ref$verifyUserAfterT = _ref.verifyUserAfterToken,
        verifyUserAfterToken = _ref$verifyUserAfterT === undefined ? false : _ref$verifyUserAfterT;

    _classCallCheck(this, MagicLinkStrategy);

    if (!secret) throw new Error('Magic Link authentication strategy requires an encryption secret');
    if (!userFields || !userFields.length) throw new Error('Magic Link authentication strategy requires an array of mandatory user fields');
    if (!tokenField) throw new Error('Magic Link authentication strategy requires a token field');
    if (!sendToken) throw new Error('Magic Link authentication strategy requires a sendToken function');
    if (!verifyUser) throw new Error('Magic Link authentication strategy requires a verifyUser function');

    var _this = _possibleConstructorReturn(this, (MagicLinkStrategy.__proto__ || Object.getPrototypeOf(MagicLinkStrategy)).call(this));

    _this.name = 'magiclink';
    _this.secret = secret;
    _this.ttl = ttl;
    _this.passReqToCallbacks = passReqToCallbacks;
    _this.verifyUserAfterToken = verifyUserAfterToken;
    _this.userFields = userFields;
    _this.tokenField = tokenField;
    _this.sendToken = sendToken;
    _this.verifyUser = verifyUser;
    return _this;
  }

  _createClass(MagicLinkStrategy, [{
    key: 'authenticate',
    value: function () {
      var _ref2 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee(req) {
        var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
        var sanitizedOptions;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                sanitizedOptions = _extends({
                  action: 'acceptToken'
                }, options);

                // Request token logic
                // ====================================

                if (!(sanitizedOptions.action === 'requestToken')) {
                  _context.next = 3;
                  break;
                }

                return _context.abrupt('return', this.requestToken(req, sanitizedOptions));

              case 3:
                if (!(sanitizedOptions.action === 'acceptToken')) {
                  _context.next = 5;
                  break;
                }

                return _context.abrupt('return', this.acceptToken(req, sanitizedOptions));

              case 5:
                return _context.abrupt('return', this.error(new Error('Unknown action')));

              case 6:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function authenticate(_x2) {
        return _ref2.apply(this, arguments);
      }

      return authenticate;
    }()
  }, {
    key: 'requestToken',
    value: function () {
      var _ref3 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee2(req, options) {
        var userFields, user, i, fieldValue, createToken, token;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                userFields = {};
                user = void 0;
                i = 0;

              case 3:
                if (!(i < this.userFields.length)) {
                  _context2.next = 12;
                  break;
                }

                fieldValue = options.allowPost ? lookup(req.body, this.userFields[i]) || lookup(req.query, this.userFields[i]) : lookup(req.query, this.userFields[i]);

                if (fieldValue) {
                  _context2.next = 8;
                  break;
                }

                userFields = null;
                return _context2.abrupt('break', 12);

              case 8:
                userFields[this.userFields[i]] = fieldValue;

              case 9:
                i++;
                _context2.next = 3;
                break;

              case 12:
                if (userFields) {
                  _context2.next = 14;
                  break;
                }

                return _context2.abrupt('return', this.fail(new Error('Mandatory user fields missing')));

              case 14:
                if (this.verifyUserAfterToken) {
                  _context2.next = 34;
                  break;
                }

                _context2.prev = 15;

                if (!this.passReqToCallbacks) {
                  _context2.next = 22;
                  break;
                }

                _context2.next = 19;
                return this.verifyUser(req, userFields);

              case 19:
                user = _context2.sent;
                _context2.next = 25;
                break;

              case 22:
                _context2.next = 24;
                return this.verifyUser(userFields);

              case 24:
                user = _context2.sent;

              case 25:
                _context2.next = 30;
                break;

              case 27:
                _context2.prev = 27;
                _context2.t0 = _context2['catch'](15);
                return _context2.abrupt('return', this.error(_context2.t0));

              case 30:
                if (user) {
                  _context2.next = 32;
                  break;
                }

                return _context2.abrupt('return', this.fail(new Error(options.authMessage || 'No user found'), 400));

              case 32:
                _context2.next = 35;
                break;

              case 34:
                user = userFields;

              case 35:

                // Generate JWT
                createToken = promisify(jwt.sign);
                token = void 0;
                _context2.prev = 37;
                _context2.next = 40;
                return createToken({ user: user, iat: Math.floor(Date.now() / 1000) }, this.secret, { expiresIn: this.ttl });

              case 40:
                token = _context2.sent;
                _context2.next = 46;
                break;

              case 43:
                _context2.prev = 43;
                _context2.t1 = _context2['catch'](37);
                return _context2.abrupt('return', this.error(_context2.t1));

              case 46:
                _context2.prev = 46;

                if (!this.passReqToCallbacks) {
                  _context2.next = 52;
                  break;
                }

                _context2.next = 50;
                return this.sendToken(req, user, token);

              case 50:
                _context2.next = 54;
                break;

              case 52:
                _context2.next = 54;
                return this.sendToken(user, token);

              case 54:
                _context2.next = 59;
                break;

              case 56:
                _context2.prev = 56;
                _context2.t2 = _context2['catch'](46);
                return _context2.abrupt('return', this.error(_context2.t2));

              case 59:
                return _context2.abrupt('return', this.pass({ message: 'Token succesfully delivered' }));

              case 60:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this, [[15, 27], [37, 43], [46, 56]]);
      }));

      function requestToken(_x3, _x4) {
        return _ref3.apply(this, arguments);
      }

      return requestToken;
    }()
  }, {
    key: 'acceptToken',
    value: function () {
      var _ref4 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee3(req, options) {
        var token, user, verifyToken, _ref5, tokenUser;

        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                token = options.allowPost ? lookup(req.body, this.tokenField) || lookup(req.query, this.tokenField) : lookup(req.query, this.tokenField);

                if (token) {
                  _context3.next = 3;
                  break;
                }

                return _context3.abrupt('return', this.pass({ message: 'Token missing' }));

              case 3:
                user = void 0;
                _context3.prev = 4;

                // Verify token
                verifyToken = promisify(jwt.verify);
                _context3.next = 8;
                return verifyToken(token, this.secret);

              case 8:
                _ref5 = _context3.sent;
                tokenUser = _ref5.user;

                user = tokenUser;
                _context3.next = 16;
                break;

              case 13:
                _context3.prev = 13;
                _context3.t0 = _context3['catch'](4);
                return _context3.abrupt('return', this.fail(_context3.t0));

              case 16:
                if (!this.verifyUserAfterToken) {
                  _context3.next = 34;
                  break;
                }

                _context3.prev = 17;

                if (!this.passReqToCallbacks) {
                  _context3.next = 24;
                  break;
                }

                _context3.next = 21;
                return this.verifyUser(req, user);

              case 21:
                user = _context3.sent;
                _context3.next = 27;
                break;

              case 24:
                _context3.next = 26;
                return this.verifyUser(user);

              case 26:
                user = _context3.sent;

              case 27:
                _context3.next = 32;
                break;

              case 29:
                _context3.prev = 29;
                _context3.t1 = _context3['catch'](17);
                return _context3.abrupt('return', this.error(_context3.t1));

              case 32:
                if (user) {
                  _context3.next = 34;
                  break;
                }

                return _context3.abrupt('return', this.fail(new Error(options.authMessage || 'No user found'), 400));

              case 34:
                return _context3.abrupt('return', this.success(user));

              case 35:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this, [[4, 13], [17, 29]]);
      }));

      function acceptToken(_x5, _x6) {
        return _ref4.apply(this, arguments);
      }

      return acceptToken;
    }()
  }]);

  return MagicLinkStrategy;
}(PassportStrategy);

module.exports = MagicLinkStrategy;